﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Media.Media3D;
using System.Windows.Shapes;
using Petzold.Media3D;

namespace PseudoTransparentBackground
{
  /// <summary>
  /// Silverlight logo on a rotating dodecahedron.
  /// </summary>
  /// <remarks>
  /// Original code comes from a blog entry of Charles Petzold (http://www.charlespetzold.com/blog/2009/08/Dodecahedron-for-Silverlight-3.html)
  /// Petzold´s application will not run computers with non-english number settings, because he unfortunately uses TryParse without InvariantCulture.
  /// This issue is fixed in Petzold.Media3D.dll used with this application.
  /// </remarks>
  public partial class DodecahedronLogo : UserControl
  {
    public DodecahedronLogo()
    {
      _rotateX = new AxisAngleRotation3D { Axis = new Vector3D(1.0, 0.0, 0.0) };
      _rotateY = new AxisAngleRotation3D { Axis = new Vector3D(1.0, 2.0, 0.0) };
      _projectionLookup = new Dictionary<Transform3DGroup, Matrix3DProjection>();
      InitializeComponent();
      _polygonCount = geometry.Polygons.Count;
      _polygons = new Polygon[_polygonCount];
      _preCameraTransforms = new Transform3DGroup[_polygonCount];
      //_brush = new ImageBrush { ImageSource = new BitmapImage { UriSource = new Uri("/PseudoTransparentBackground;component/images/brush.png", UriKind.RelativeOrAbsolute) } };
      _brush = new ImageBrush { ImageSource = new BitmapImage { UriSource = new Uri("http://st-lange.net/samples/2010-05-PseudoTransparency/brush.png", UriKind.RelativeOrAbsolute) } };
      
      // Setup the twelve polygons.
      for (int i = 0; i < _polygonCount; i++)
      {
        _polygons[i] = new Polygon();
        _polygons[i].Fill = _brush;
        Polygon3D polygond = geometry.Polygons[i];
        foreach (Point point in polygond.Positions2D)
          _polygons[i].Points.Add(point);
        MatrixTransform3D transformd2 = new MatrixTransform3D();
        transformd2.Matrix = polygond.Matrix3D;
        MatrixTransform3D item = transformd2;
        Transform3DGroup key = new Transform3DGroup();
        key.Children.Add(item);
        _preCameraTransforms[i] = new Transform3DGroup();
        _preCameraTransforms[i].Children.Add(_rotateY);
        _preCameraTransforms[i].Children.Add(_rotateX);
        key.Children.Add(_preCameraTransforms[i]);
        PerspectiveCamera camera = new PerspectiveCamera();
        key.Children.Add(camera);
        Matrix3DProjection projection = new Matrix3DProjection { ProjectionMatrix = key.Value };
        key.ValueChanged += TransformGroupValueChanged;
        _projectionLookup.Add(key, projection);
        _polygons[i].Projection = projection;
        LayoutRoot.Children.Add(_polygons[i]);
      }

      RedoZorder();

      DoubleAnimation animation = new DoubleAnimation();
      animation.From = 0.0;
      animation.To = 360.0;
      animation.Duration = new Duration(TimeSpan.FromSeconds(15.0));
      animation.RepeatBehavior = RepeatBehavior.Forever;
      Storyboard.SetTarget(animation, _rotateY);
      Storyboard.SetTargetProperty(animation, new PropertyPath(AxisAngleRotation3D.AngleProperty));

      Storyboard storyboard = new Storyboard { BeginTime = TimeSpan.FromSeconds(5) };
      storyboard.Children.Add(animation);

      storyboard.Begin();
    }

    /// <summary>
    /// Rearrange the Z-order of the polygons. Because Silverlight has no 3D, but only a perspective
    /// transformation, the polygons must be reordered depending on their distance from the
    /// camera.
    /// </summary>
    private void RedoZorder()
    {
      for (int i = 0; i < _polygonCount; i++)
      {
        Matrix3D projectionMatrix = ((Matrix3DProjection)_polygons[i].Projection).ProjectionMatrix;
        Polygon3D polygon3D = geometry.Polygons[i];
        Point point = polygon3D.Center2D;
        Point3D point3D = new Point3D(point.X, point.Y, 0).Transform(projectionMatrix);
        Canvas.SetZIndex(_polygons[i], (int)((1 - point3D.Z) * 32768));
      }
    }

    private void TransformGroupValueChanged(object sender, DependencyPropertyChangedEventArgs args)
    {
      Transform3DGroup group3D = (Transform3DGroup)sender;
      Matrix3D matrix3D = group3D.Value;
      _projectionLookup[group3D].ProjectionMatrix = matrix3D;
      RedoZorder();
    }

    readonly Brush _brush;
    readonly Polygon[] _polygons;
    readonly int _polygonCount;
    readonly Transform3DGroup[] _preCameraTransforms;
    readonly Dictionary<Transform3DGroup, Matrix3DProjection> _projectionLookup;
    readonly AxisAngleRotation3D _rotateX;
    readonly AxisAngleRotation3D _rotateY;
  }
}

