﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Browser;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace PseudoTransparentBackground
{
  public partial class MainPage : UserControl
  {
    public MainPage()
    {
      // Set hard-coded start postion from html file.
      LastAppPosion = new Point(90, 50);

      InitializeComponent();

      // Animate the Silverlight application.
      _appPosition = new AppPosition();
      _appAnimationStoryboard = new Storyboard { BeginTime = TimeSpan.FromSeconds(20) };
      _appAnimationStoryboard.Completed += Animation_Completed;

      // Setup the Left position animation.
      _leftAnimation = new DoubleAnimation();
      _leftAnimation.EasingFunction = new PowerEase { EasingMode = EasingMode.EaseInOut };
      Storyboard.SetTarget(_leftAnimation, _appPosition);
      Storyboard.SetTargetProperty(_leftAnimation, new PropertyPath("Left"));
      _appAnimationStoryboard.Children.Add(_leftAnimation);

      // Setup the Top position animation.
      _topAnimation = new DoubleAnimation();
      _topAnimation.EasingFunction = new PowerEase { EasingMode = EasingMode.EaseInOut };
      Storyboard.SetTarget(_topAnimation, _appPosition);
      Storyboard.SetTargetProperty(_topAnimation, new PropertyPath("Top"));
      _appAnimationStoryboard.Children.Add(_topAnimation);

      AnimateAppPosition();

      // Animate the dotted line that borders the Silverlight application.
      _dottedLineStoryboard = new Storyboard();
      var dotAnimation = new DoubleAnimation();
      dotAnimation.From = 2;
      dotAnimation.To = 0;
      dotAnimation.Duration = new Duration(TimeSpan.FromMilliseconds(1000));
      dotAnimation.RepeatBehavior = RepeatBehavior.Forever;
      Storyboard.SetTarget(dotAnimation, ControlBorder);
      Storyboard.SetTargetProperty(dotAnimation, new PropertyPath("StrokeDashOffset"));
      _dottedLineStoryboard.Children.Add(dotAnimation);
      _dottedLineStoryboard.Begin();
    }

    void AnimateAppPosition()
    {
      const int pixelsPerSecond = 40;
      const int minX = 10;
      const int maxX = 600 - 150 - 10;
      const int minY = 10;
      const int maxY = 375 - 150 - 10;

      var newPosition = new Point(_rand.Next(minX, maxX), _rand.Next(minY, maxY));
      double dx = newPosition.X - LastAppPosion.X;
      double dy = newPosition.Y - LastAppPosion.Y;
      var distance = Math.Sqrt(dx * dx + dy * dy);

      // Normalize speed.
      double ms = distance / pixelsPerSecond * 1000;

      _leftAnimation.From = LastAppPosion.X;
      _leftAnimation.To = newPosition.X;
      _leftAnimation.Duration = new Duration(TimeSpan.FromMilliseconds(ms));

      _topAnimation.From = LastAppPosion.Y;
      _topAnimation.To = newPosition.Y;
      _topAnimation.Duration = new Duration(TimeSpan.FromMilliseconds(ms));

      LastAppPosion = newPosition;
      _appAnimationStoryboard.Begin();
    }

    void Animation_Completed(object sender, EventArgs e)
    {
      _appAnimationStoryboard.BeginTime = TimeSpan.Zero;
      AnimateAppPosition();
    }

    protected override void OnMouseLeftButtonDown(MouseButtonEventArgs e)
    {
      ControlBorder.Visibility = ControlBorder.Visibility == Visibility.Visible ? Visibility.Collapsed : Visibility.Visible;
    }

    Point LastAppPosion { get; set; }

    readonly Random _rand = new Random();
    readonly Storyboard _dottedLineStoryboard;
    readonly Storyboard _appAnimationStoryboard;
    readonly AppPosition _appPosition;
    readonly DoubleAnimation _leftAnimation;
    readonly DoubleAnimation _topAnimation;
  }
}
